# configuration for this product: external file system paths etc.

# (c) 2012 oeko.net
# c/o toni mueller <support@oeko.net>
# license: GPLv3


ON_VIDEO_FS_PATH = u'/tmp'

# BEWARE: URLs must not contain Unicode!
ON_VIDEO_URL = "http://localhost/"

PLAYER = "flowplayer-3.2.5.swf"

MAX_WIDTH = 750
DEFAULT_WIDTH = 720
DEFAULT_HEIGHT = 540

MIN_HEIGHT = 100
MAX_HEIGHT = 900
